import 'dart:collection';
import 'package:aitrainer_app/localization/app_language.dart';
import 'package:aitrainer_app/model/cache.dart';
import 'package:aitrainer_app/model/exercise_ability.dart';
import 'package:aitrainer_app/model/exercise_tree.dart';
import 'package:aitrainer_app/model/exercise_type.dart';
import 'package:aitrainer_app/model/workout_menu_tree.dart';
import 'package:aitrainer_app/repository/exercise_repository.dart';
import 'package:aitrainer_app/service/exercise_tree_service.dart';
import 'package:aitrainer_app/service/exercise_type_service.dart';
import 'package:aitrainer_app/service/logging.dart';
import 'package:flutter/material.dart';

class Antagonist {
  static String chest = "Chest";
  static int chestNr = 1;
  static String biceps = "Biceps";
  static int bicepsNr = 2;
  static String triceps = "Triceps";
  static int tricepsNr = 3;
  static String back = "Back";
  static int backNr = 4;
  static String shoulder = "Shoulders";
  static int shoulderNr = 5;
  static String core = "Core & ABS";
  static int coreNr = 6;
  static String thigh = "Thigh";
  static int thighNr = 7;
  static String calf = "Calf";
  static int calfNr = 8;
}

class WorkoutTreeRepository with Logging {
  final LinkedHashMap tree = LinkedHashMap<String, WorkoutMenuTree>();
  SplayTreeMap sortedTree = SplayTreeMap<String, List<WorkoutMenuTree>>();
  bool isEnglish;
  WorkoutType workoutType;

  final Map<String, int> _antagonist = {
    Antagonist.chest: Antagonist.chestNr,
    Antagonist.biceps: Antagonist.bicepsNr,
    Antagonist.triceps: Antagonist.tricepsNr,
    Antagonist.back: Antagonist.backNr,
    Antagonist.shoulder: Antagonist.shoulderNr,
    Antagonist.core: Antagonist.coreNr,
    Antagonist.thigh: Antagonist.thighNr,
    Antagonist.calf: Antagonist.calfNr
  };

  Future<void> createTree() async {
    //if (Cache().getExerciseTree().length > 0 || Cache().getWorkoutMenuTree().length > 0) return;
    isEnglish = AppLanguage().appLocal == Locale('en');
    log("** Start creating tree on lang: " +
        AppLanguage().appLocal.languageCode +
        " tree length: " +
        Cache().getExerciseTree().length.toString());

    List<ExerciseTree> exerciseTree = Cache().getExerciseTree();
    if (exerciseTree == null || exerciseTree.length == 0) {
      exerciseTree = await ExerciseTreeApi().getExerciseTree();
    }

    List<ExerciseType> exerciseTypes = Cache().getExerciseTypes();
    if (exerciseTypes == null || exerciseTypes.length == 0) {
      exerciseTypes = await ExerciseTypeApi().getExerciseTypes();
    }

    exerciseTree.forEach((treeItem) async {
      //log(" -- TreeItem " + treeItem.toJson().toString() + " active " + treeItem.active.toString());
      if (treeItem.active == true) {
        String treeName = isEnglish ? treeItem.name : treeItem.nameTranslation;
        //String assetImage = await _buildImage(treeItem.imageUrl);

        bool is1RM = treeItem.name == 'One Rep Max' ? true : false;
        if (is1RM == false && treeItem.parentId != 0) {
          is1RM = isParent1RM(treeItem.parentId);
        }

        bool isEndurance = treeItem.name == 'Endurance' ? true : false;
        if (isEndurance == false && treeItem.parentId != 0) {
          isEndurance = isParentEndurance(treeItem.parentId);
        }

        bool isRunning = treeItem.name == "Cardio" ? true : false;
        if (isRunning == false && treeItem.parentId != 0) {
          isRunning = isParentRunning(treeItem.parentId);
        }

        WorkoutMenuTree menuItem = WorkoutMenuTree(
          treeItem.treeId,
          treeItem.parentId,
          treeName,
          treeItem.imageUrl,
          Colors.white,
          30,
          false,
          0,
          null,
          false,
          is1RM,
          isEndurance,
          isRunning,
          treeItem.name,
        );
        menuItem = this.setWorkoutTypes(menuItem, treeItem);
        this.tree[treeItem.name + "_" + treeItem.parentId.toString()] = menuItem;
        //log("WorkoutMenuTree item " + menuItem.toJson().toString());
      }
    });

    exerciseTypes.forEach((exerciseType) async {
      if (!(exerciseType.imageUrl.isEmpty || exerciseType.name.isEmpty || exerciseType.nameTranslation.isEmpty) &&
          exerciseType.active == true) {
        String exerciseTypeName = isEnglish ? exerciseType.name : exerciseType.nameTranslation;
        //String assetImage = await _buildImage(exerciseType.imageUrl); //'asset/menu/' + exerciseType.imageUrl.substring(7);
        if (exerciseType.parents.isNotEmpty) {
          exerciseType.parents.forEach((parentId) {
            bool is1RM = this.isParent1RM(parentId);
            bool isEndurance = this.isParentEndurance(parentId);
            if (is1RM) exerciseType.setAbility(ExerciseAbility.oneRepMax);
            if (isEndurance) exerciseType.setAbility(ExerciseAbility.endurance);
            bool isRunning = this.isParentRunning(parentId);
            if (isRunning) exerciseType.setAbility(ExerciseAbility.running);
            WorkoutMenuTree menuItem = WorkoutMenuTree(
                exerciseType.exerciseTypeId,
                parentId,
                exerciseTypeName,
                exerciseType.imageUrl,
                Colors.white,
                24,
                true,
                exerciseType.exerciseTypeId,
                exerciseType,
                exerciseType.base,
                is1RM,
                isEndurance,
                isRunning,
                exerciseType.name);
            this.tree[exerciseType.name] = menuItem;
            //log("WorkoutMenuTree item " + menuItem.toJson().toString());
            /* log("ExerciseType in Menu item " +
                exerciseType.toJson().toString() +
                " is1RM: " +
                is1RM.toString() +
                " isEndurance: " +
                isEndurance.toString()); */
          });
        } else {
          //log("No Parents " + exerciseType.toJson().toString());
        }
      } else {
        //log("ExerciseType missing data: " + exerciseType.exerciseTypeId.toString());
      }
    });

    Cache().setWorkoutMenuTree(tree);
    ExerciseRepository exerciseRepository = ExerciseRepository();
    exerciseRepository.getBaseExerciseFinishedPercent();
  }

  WorkoutMenuTree setWorkoutTypes(WorkoutMenuTree menu, ExerciseTree treeItem) {
    WorkoutType.values.forEach((workoutType) {
      if (workoutType.equalsString(treeItem.name) || existWorkoutTypeInTree(workoutType)) {
        menu.workoutTypes.add(workoutType);
      }
    });
    return menu;
  }

  bool existWorkoutTypeInTree(WorkoutType workoutType) {
    bool exists = false;
    this.tree.forEach((key, treeItem) {
      if (workoutType.equalsString(treeItem.name)) {
        exists = true;
      }
    });
    return exists;
  }

  bool isParentRunning(int treeId) {
    bool isTreeItemRunning = false;

    this.tree.forEach((key, value) {
      WorkoutMenuTree treeItem = value as WorkoutMenuTree;
      if (treeItem.id == treeId) {
        isTreeItemRunning = isTreeItemRunning || treeItem.isRunning;
        //log (treeItem.name + " 1RM " + treeItem.is1RM.toString() );
      }
    });

    return isTreeItemRunning;
  }

  bool isParent1RM(int treeId) {
    bool isTreeItem1RM = false;

    this.tree.forEach((key, value) {
      WorkoutMenuTree treeItem = value as WorkoutMenuTree;
      if (treeItem.id == treeId) {
        isTreeItem1RM = isTreeItem1RM || treeItem.is1RM;
        //log (treeItem.name + " 1RM " + treeItem.is1RM.toString() );
      }
    });

    return isTreeItem1RM;
  }

  bool isParentEndurance(int treeId) {
    bool isTreeItemEndurance = false;

    this.tree.forEach((key, value) {
      WorkoutMenuTree treeItem = value as WorkoutMenuTree;
      if (treeItem.id == treeId) {
        isTreeItemEndurance = isTreeItemEndurance || treeItem.isEndurance;
        //log(treeItem.id.toString() + " " + treeItem.name + " Endurance? " + treeItem.isEndurance.toString());
      }
    });

    return isTreeItemEndurance;
  }

  bool isChild(int parentId) {
    bool isChild = true;

    this.getBranch(parentId).forEach((key, value) {
      WorkoutMenuTree workoutTree = value;
      isChild = isChild && workoutTree.child;
    });
    //log("Check child " + parentId.toString() + " child: " + isChild.toString());
    return isChild;
  }

  bool isChildAndGym(int parentId) {
    bool isChild = true;
    bool isGym = true;

    this.getBranch(parentId).forEach((key, value) {
      WorkoutMenuTree workoutTree = value;
      isChild = isChild && workoutTree.child;
      isGym = isGym && (workoutTree.is1RM || workoutTree.isEndurance);
    });
    return isChild && isGym;
  }

  LinkedHashMap<String, WorkoutMenuTree> getBranch(int parent, {bool filtering = false}) {
    LinkedHashMap<String, WorkoutMenuTree> branch = LinkedHashMap<String, WorkoutMenuTree>();
    tree.forEach((key, value) {
      WorkoutMenuTree workoutTree = value as WorkoutMenuTree;
      if (parent == workoutTree.parent) {
        branch[key] = value;
      }
    });
    return branch;
  }

  List<WorkoutMenuTree> getBranchList(int parent) {
    List branch = List<WorkoutMenuTree>();
    tree.forEach((key, value) {
      WorkoutMenuTree workoutTree = value as WorkoutMenuTree;
      if (parent == workoutTree.parent) {
        branch.add(workoutTree);
      }
    });
    return branch;
  }

  WorkoutMenuTree getParentItem(int parent) {
    WorkoutMenuTree parentItem;
    tree.forEach((key, value) {
      WorkoutMenuTree workoutTree = value as WorkoutMenuTree;
      if (parent == workoutTree.id) {
        parentItem = workoutTree;
      }
    });
    return parentItem;
  }

  void sortByMuscleType() {
    sortedTree = SplayTreeMap<String, List<WorkoutMenuTree>>();
    tree.forEach((key, value) {
      WorkoutMenuTree workoutTree = value as WorkoutMenuTree;
      if (workoutTree.nameEnglish != 'One Rep Max' && workoutTree.is1RM && workoutTree.exerciseTypeId == 0) {
        String treeName = _antagonist[workoutTree.nameEnglish].toString() + ". " + workoutTree.name;
        sortedTree[treeName] = this.getBranchList(workoutTree.id);
      }
    });

    return;
  }

  int getMissingTreeIdByName(String name) {
    int missingId = 0;
    tree.forEach((key, value) {
      WorkoutMenuTree item = value as WorkoutMenuTree;
      if (item.name == name || name == item.nameEnglish) {
        missingId = item.id;
      }
    });
    return missingId;
  }
}
